# %%
import numpy as np
import pandas as pd
import json
import urllib
import matplotlib.pyplot as plt

from scipy.stats import exponweib
from scipy.optimize import fmin

def fitweibull(x):
   def optfun(theta):
      return -np.sum(np.log(exponweib.pdf(x, 1, theta[0], scale = theta[1], loc = 0)))
  
   logx = np.log(x)
   shape = 1.2 / np.std(logx)
   scale = np.exp(np.mean(logx) + (0.572 / shape))
   fmin(optfun, [shape, scale], xtol = 0.01, ftol = 0.01, disp = 0)
   return shape, scale


# %%
# get wind data using api which returns json

url = 'https://archive-api.open-meteo.com/v1/archive?latitude=48.0104&longitude=7.9013&start_date=2022-01-01&end_date=2022-12-31&hourly=windspeed_100m&windspeed_unit=ms'
response = urllib.request.urlopen(url)
data = json.loads(response.read())

wind_data = pd.DataFrame(index=data['hourly']['time'], data=data['hourly']['windspeed_100m'], columns=['wind_speed_100m'])
wind_data.index = pd.to_datetime(wind_data.index)
# %%
mean_100m = wind_data['wind_speed_100m'].mean(axis = 0)
std_100m = wind_data['wind_speed_100m'].std(axis = 0)

print('Mean wind speed at 100m:', round(mean_100m,2), 'm/s')
print('Standard deviation at 100m:', round(std_100m,2), 'm/s')

#%%
plt.figure()
plt.hist(
    wind_data['wind_speed_100m'],
    bins=range(26),
    rwidth=0.9,
    color='tab:blue',
    density=True,
)

plt.ylabel('Frequency')
plt.xlabel('Wind speed (m/s)')
plt.title('Wind speed distribution at Feldberg/Schwarzwald meteo station')

plt.show()


# %% Raylegh
U = np.linspace(0,25,26)

scale = mean_100m / np.sqrt(np.pi/2)

Rayleigh = U / scale**2 * np.exp(-U**2/(2*scale**2))

# %%
x = wind_data['wind_speed_100m'].to_list()
shape, scale = fitweibull(x)

print('k =', shape, ', c =', scale)

Weibull = (shape / scale) * (U/scale)**(shape-1) * np.exp(-(U/scale)**shape)

plt.figure()
plt.hist(
    wind_data['wind_speed_100m'],
    bins=range(26),
    rwidth=0.9,
    color='tab:blue',
    density=True,
)
plt.plot(Rayleigh, color = 'tab:red', label = 'Rayleigh')
plt.plot(Weibull, color = 'tab:green', label = 'Weibull')

plt.ylabel('Frequency')
plt.xlabel('Wind speed (m/s)')
plt.legend()
plt.title('Wind speed distribution at Feldberg/Schwarzwald meteo station')

plt.show()


# %%
# get second set of wind speed at 10m
url = 'https://archive-api.open-meteo.com/v1/archive?latitude=48.0104&longitude=7.9013&start_date=2022-01-01&end_date=2022-12-31&hourly=windspeed_10m&windspeed_unit=ms'
response = urllib.request.urlopen(url)
data = json.loads(response.read())

wind_data['wind_speed_10m'] = data['hourly']['windspeed_10m']
# %%
mean_10m = wind_data['wind_speed_10m'].mean(axis = 0)
std_10m = wind_data['wind_speed_10m'].std(axis = 0)

print('Mean wind speed at 10m:', round(mean_10m,2), 'm/s')
print('Standard deviation at 10m:', round(std_10m,2), 'm/s')

# %%
z = (mean_100m * np.log(10) - mean_10m * np.log(100)) / (mean_100m - mean_10m)
z = np.exp(z)
a = mean_10m / (np.log(10) - np.log(z))

print(f'Parameter a = {a:.2f}, Parameter z = {z:.2f}')
# %%
new_height = 80
given_height = 100
wind_data['wind_speed_80m'] = wind_data['wind_speed_100m'] * np.log((new_height-a)/z) / np.log((given_height-a)/z)
# %%
